*** makeinfo.c	Sun Jun 11 22:37:28 1995
--- makeinfo.c	Tue Jun 13 17:44:05 1995
***************
*** 29,34 ****
--- 29,42 ----
  
     Makeinfo is authored by Brian Fox (bfox@ai.mit.edu). */
  
+ /* Define RTF_OUTPUT, if you want to create RTF files. */
+ #define RTF_OUTPUT
+ 
+ #ifdef RTF_OUTPUT
+ /* Code in RTF_OUTPUT sections is Copyright (C) 1994-95 Christian Schenk
+    (cschenk@gs.detewe.de). */
+ #endif
+ 
  /* You can change some of the behaviour of Makeinfo by changing the
     following defines: */
  
***************
*** 56,61 ****
--- 64,84 ----
     newlines in the input file (i.e., one or more blank lines). */
  #define DEFAULT_PARAGRAPH_SPACING 1
  
+ #ifdef RTF_OUTPUT
+ #define _quarter_inch   (_1_inch/4)
+ #define _half_inch      (_1_inch/2)
+ #define _1_inch         1440
+ #define _2_inch         (2 * _1_inch)
+ 
+ #define RTF_FONT_NORMAL         0
+ #define RTF_FONT_FIXED_WIDTH    1
+ 
+ #define RTF_MENU_INDENTATION                    _2_inch
+ #define RTF_TABLE_INDENTATION_INCREMENT         _half_inch
+ #define RTF_ITEMIZE_INDENTATION_INCREMENT       _half_inch
+ #define RTF_ENUMERATE_INDENTATION_INCREMENT     _half_inch
+ #endif
+ 
  /* **************************************************************** */
  /*								    */
  /*			Include File Declarations       	    */
***************
*** 192,198 ****
--- 215,225 ----
  /* Stuff for splitting large files. */
  #define SPLIT_SIZE_THRESHOLD 70000  /* What's good enough for Stallman... */
  #define DEFAULT_SPLIT_SIZE 50000    /* Is probably good enough for me. */
+ #ifdef RTF_OUTPUT
+ int splitting = 0;		    /* We want a single RTF file. */
+ #else
  int splitting = 1;		    /* Always true for now. */
+ #endif
  
  typedef int FUNCTION ();	/* So I can say FUNCTION *foo; */
  
***************
*** 277,287 ****
--- 304,329 ----
  int indented_fill = 0;
  
  /* The column at which long lines are broken. */
+ #ifdef RTF_OUTPUT
+ int fill_column = 30000;	/* Do not break lines. */
+ #else
  int fill_column = 72;
+ #endif
  
  /* The amount of indentation to apply at the start of each line. */
  int current_indent = 0;
  
+ #ifdef RTF_OUTPUT
+ int rtf_font = 0;               /* font number */
+ int rtf_current_font = 0;       /*  */
+ int rtf_li = 0;                 /* left indent */
+ int rtf_fi = 0;                 /* first indent */
+ int rtf_fs = 20;                /* font size */
+ int rtf_tab_stops[10];          /* tab stop positions */
+ int rtf_n_tab_stops = 0;        /* number of tab stops */
+ int in_non_scrolling_region = 0;
+ #endif
+ 
  /* The amount of indentation to add at the starts of paragraphs.
     0 means don't change existing indentation at paragraph starts.
     > 0 is amount to indent new paragraphs by.
***************
*** 750,755 ****
--- 792,802 ----
  int major_version = 1;
  int minor_version = 55;
  
+ #if defined (RTF_OUTPUT)
+ int rtf_major_version = 1;
+ int rtf_minor_version = 2;
+ #endif
+ 
  struct option long_options[] =
  {
    { "error-limit", 1, 0, 'e' },			/* formerly -el */
***************
*** 889,896 ****
--- 936,950 ----
  /* Display the version info of this invocation of Makeinfo. */
  print_version_info ()
  {
+ #ifdef RTF_OUTPUT
+   fprintf (stderr,
+ 	   "This is Makertf version %d.%d (GNU Makeinfo version %d.%d).\n",
+ 	   rtf_major_version, rtf_minor_version,
+ 	   major_version, minor_version);
+ #else
    fprintf (stderr, "This is GNU Makeinfo version %d.%d.\n",
  	   major_version, minor_version);
+ #endif
  }
  
  /* **************************************************************** */
***************
*** 948,953 ****
--- 1002,1043 ----
  /* Tell the user how to use this program. */
  usage ()
  {
+ #ifdef RTF_OUTPUT
+   fprintf (stderr, "Usage: %s [options] texinfo-file...\n\
+ \n\
+ This program accepts as input files of texinfo commands and text\n\
+ and outputs an RTF file.\n\
+ \n\
+ The options are:\n\
+ `-I DIR'              to add DIR to the directory search list for including\n\
+ 		      files with the `@include' command.\n\
+ -D VAR                to define a variable, as with `@set'.\n\
+ -U VAR                to undefine a variable, as with `@clear'.\n\
+ `--no-validate'       to suppress node cross reference validation.\n\
+ `--no-warn'           to suppress warning messages (errors are still output).\n\
+ `--no-split'          to suppress the splitting of large files.\n\
+ `--no-headers'        to suppress the output of Node: Foo headers.\n\
+ `--verbose'           to print information about what is being done.\n\
+ `--version'           to print the version number of Makeinfo.\n\
+ `--output FILE' or `-o FILE'\n\
+ 		      to specify the output file.  When you specify the\n\
+ 		      output file in this way, any `@setfilename' in the\n\
+ 		      input file is ignored.\n\
+ `--paragraph-indent NUM'\n\
+ 		      to set the paragraph indent to NUM (default %d).\n\
+ `--fill-column NUM'   to set the filling column to NUM (default %d).\n\
+ `--error-limit NUM'   to set the error limit to NUM (default %d).\n\
+ `--reference-limit NUM'\n\
+ 		      to set the reference warning limit to NUM (default %d).\n\
+ `--footnote-style STYLE'\n\
+ 		      to set the footnote style to STYLE.  STYLE should\n\
+ 		      either be `separate' to place footnotes in their own\n\
+ 		      node, or `end', to place the footnotes at the end of\n\
+ 		      the node in which they are defined (the default).\n\n",
+ 	   progname, paragraph_start_indent,
+ 	   fill_column, max_error_level, reference_warning_limit);
+ 
+ #else
    fprintf (stderr, "Usage: %s [options] texinfo-file...\n\
  \n\
  This program accepts as input files of texinfo commands and text\n\
***************
*** 981,986 ****
--- 1071,1078 ----
                        the node in which they are defined (the default).\n\n",
  	   progname, paragraph_start_indent,
  	   fill_column, max_error_level, reference_warning_limit);
+ 
+ #endif
    exit (FATAL);
  }
  
***************
*** 1040,1046 ****
--- 1132,1142 ----
  
    filename = fullpath;
  
+ #ifdef __MSDOS__
+   file = open (filename, O_RDONLY | O_TEXT); /* allows DOS-Edited cr/lf'ed */
+ #else                                        /* files to be read (HBB) */
    file = open (filename, O_RDONLY);
+ #endif
    if (file < 0)
      goto error_exit;
  
***************
*** 1051,1064 ****
       readable bytes is always less than this value.  The arcane
       mysteries of VMS/RMS are too much to probe, so this hack
      suffices to make things work. */
! #if defined (VMS)
    while ((n = read (file, result + count, fileinfo.st_size)) > 0)
      count += n;
    if (n == -1)
! #else /* !VMS */
      count = fileinfo.st_size;
      if (read (file, result, fileinfo.st_size) != fileinfo.st_size)
! #endif /* !VMS */
    error_exit:
      {
        if (result)
--- 1147,1165 ----
       readable bytes is always less than this value.  The arcane
       mysteries of VMS/RMS are too much to probe, so this hack
      suffices to make things work. */
! #if defined (VMS) || defined (__MSDOS__)
    while ((n = read (file, result + count, fileinfo.st_size)) > 0)
      count += n;
    if (n == -1)
! #else /* !VMS || __MSDOS__ */
! #ifdef RTF_OUTPUT
!   count = read (file, result, fileinfo.st_size);
!   if (count < 0)
! #else /* !RTF_OUTPUT */
      count = fileinfo.st_size;
      if (read (file, result, fileinfo.st_size) != fileinfo.st_size)
! #endif /* !RTF_OUTPUT */
! #endif /* !VMS || __MSDOS__ */
    error_exit:
      {
        if (result)
***************
*** 1784,1790 ****
--- 1885,1895 ----
      }
  
    canon_white (output_filename);
+ #ifdef RTF_OUTPUT
+   printf ("Making RTF file `%s' from `%s'.\n", output_filename, name);
+ #else
    printf ("Making info file `%s' from `%s'.\n", output_filename, name);
+ #endif
  
    if (verbose_mode)
      fprintf (stderr, "  The input file contains %d characters.\n",
***************
*** 1800,1806 ****
--- 1905,1915 ----
        if (!real_output_filename)
  	real_output_filename = expand_filename (output_filename, name);
  
+ #ifdef __MSDOS__
+       output_stream = fopen (real_output_filename, "wb");
+ #else
        output_stream = fopen (real_output_filename, "w");
+ #endif
      }
  
    if (output_stream == NULL)
***************
*** 1825,1836 ****
--- 1934,1949 ----
  	line_number++;
    }
  
+ #ifdef RTF_OUTPUT
+   start_rtf_file ();
+ #else
    if (!no_headers)
      {
        add_word_args ("This is Info file %s, produced by Makeinfo-%d.%d from ",
  		     output_filename, major_version, minor_version);
        add_word_args ("the input file %s.\n", input_filename);
      }
+ #endif
  
    close_paragraph ();
    reader_loop ();
***************
*** 1845,1854 ****
--- 1958,1974 ----
        if (tag_table != NULL)
  	{
  	  tag_table = (TAG_ENTRY *) reverse_list (tag_table);
+ #ifdef RTF_OUTPUT
+ #else
  	  if (!no_headers)
  	    write_tag_table ();
+ #endif
  	}
  
+ #ifdef RTF_OUTPUT
+       end_rtf_file ();
+ #endif
+ 
        if (output_stream != stdout)
  	fclose (output_stream);
  
***************
*** 1981,1986 ****
--- 2101,2112 ----
  		 gauranteed to be NULL when glean_node_from_menu () is
  		 called with a non-zero argument. */
  	      tem = glean_node_from_menu (1);
+ #ifdef RTF_OUTPUT
+ 	      add_char ('\n');
+ 	      input_text_offset++;
+ 	      process_menu_line ();
+ 	      continue;
+ #endif
  	    }
  	}
  
***************
*** 2282,2288 ****
--- 2408,2421 ----
      case '\n':
        if (!filling_enabled)
  	{
+ #ifdef RTF_OUTPUT
+ 	  if (in_menu || in_non_scrolling_region)
+ 	    inserts ("\\par\n");
+ 	  else
+ 	    inserts ("\\line\n");
+ #else
  	  insert ('\n');
+ #endif
  
  	  if (force_flush_right)
  	    {
***************
*** 2361,2366 ****
--- 2494,2502 ----
  			while (temp && whitespace (output_paragraph[temp - 1]))
  			  temp--;
  
+ #ifdef RTF_OUTPUTX
+ 			output_paragraph[temp++] = ' ';
+ #endif
  			output_paragraph[temp++] = '\n';
  
  			/* We have correctly broken the line where we want
***************
*** 2391,2396 ****
--- 2527,2534 ----
  			    }
  			}
  
+ #ifdef RTF_OUTPUT
+ #else
  			/* Filled, but now indent if that is right. */
  			if (indented_fill && current_indent)
  			  {
***************
*** 2433,2438 ****
--- 2571,2577 ----
  			    output_paragraph_offset += current_indent;
  			    free (temp_buffer);
  			  }
+ #endif
  			output_column = 0;
  			while (temp < output_paragraph_offset)
  			  output_column +=
***************
*** 2446,2452 ****
--- 2585,2599 ----
  
  	if (!suppress_insert)
  	  {
+ #ifdef RTF_OUTPUT
+ 	    if (character == '{' || character == '}' || character == '\\'
+ 		|| ! isascii (character))
+ 	      vinserts ("\\'%02X", (int) (character & 0xff));
+ 	    else
  	    insert (character);
+ #else
+ 	    insert (character);
+ #endif
  	    last_inserted_character = character;
  	  }
  	last_char_was_newline = 0;
***************
*** 2624,2637 ****
--- 2771,2807 ----
        output_paragraph_offset = ++tindex;
  
        /* Break the line if that is appropriate. */
+ #ifdef RTF_OUTPUT      
+       if (paragraph_spacing >= 0) 
+ 	{
+ 	  if (output_paragraph_offset >= 5
+ 	      && strncmp (&output_paragraph[output_paragraph_offset-5],
+ 			  "\\line", 5) == 0)
+ 	    {
+ 	      output_paragraph_offset -= 5;
+ 	    }
+ 	  if (! in_non_scrolling_region)
+ 	    inserts ("\\par");
+ 	  insert ('\n');
+ 	}
+ #else
        if (paragraph_spacing >= 0)
  	insert ('\n');
+ #endif
  
        /* Add as many blank lines as is specified in PARAGRAPH_SPACING. */
        if (!force_flush_right)
  	{
+ #ifdef RTF_OUTPUT
+ 	  if (! in_non_scrolling_region)
+ 	    {
+ 	      for (i = 0; i < (paragraph_spacing - line_already_broken); i++)
+ 		inserts ("\\par\n");
+ 	    }
+ #else
  	  for (i = 0; i < (paragraph_spacing - line_already_broken); i++)
  	    insert ('\n');
+ #endif
  	}
  
        /* If we are doing flush right indentation, then do it now
***************
*** 2729,2734 ****
--- 2899,2926 ----
  	      indent (amount_to_indent);
  	      output_column += amount_to_indent;
  	    }
+ 
+ #ifdef RTF_OUTPUT
+ 	  /* rtf paragraph properties */
+ 	  vinserts ("\\pard");
+ 	  if (! filling_enabled && in_fixed_width_font)
+ 	    inserts ("\\keep");
+ 	  if (rtf_fi != 0)
+ 	    vinserts ("\\fi%d", rtf_fi);
+ 	  if (rtf_li != 0)
+ 	    vinserts ("\\li%d", rtf_li);
+ 	  {
+ 	    int i;
+ 	    for (i=0; i<rtf_n_tab_stops; i++)
+ 	      vinserts ("\\tx%d", rtf_tab_stops[i]);
+ 	  }
+ 	  if (rtf_current_font != rtf_font)
+ 	    {
+ 	      rtf_current_font = rtf_font;
+ 	      vinserts ("\\f%d", rtf_font);
+ 	    }
+ 	  inserts ("{}");
+ #endif
  	}
      }
    else
***************
*** 2750,2757 ****
--- 2942,2952 ----
        elt = elt->next;
      }
  
+ #ifdef RTF_OUTPUT
+ #else
    while (--amount >= 0)
      insert (' ');
+ #endif
  }
  
  /* Search forward for STRING in input_text.
***************
*** 2771,2776 ****
--- 2966,2972 ----
    return (-1);
  }
  
+ #ifndef __MSDOS__
  /* Whoops, Unix doesn't have stricmp. */
  
  /* Case independent string compare. */
***************
*** 2794,2799 ****
--- 2990,2996 ----
  	return (ch1 - ch2);
      }
  }
+ #endif
  
  enum insertion_type { menu, quotation, lisp, smalllisp, example,
    smallexample, display, itemize, format, enumerate, cartouche, table,
***************
*** 2821,2826 ****
--- 3018,3031 ----
    int indented_fill;
    enum insertion_type insertion;
    int inhibited;
+ #ifdef RTF_OUTPUT
+   int rtf_li;
+   int rtf_fi;
+   int rtf_tab_stops[10];
+   int rtf_n_tab_stops;
+   int rtf_font;
+   int rtf_fs;
+ #endif
  } INSERTION_ELT;
  
  INSERTION_ELT *insertion_stack = (INSERTION_ELT *) NULL;
***************
*** 2899,2904 ****
--- 3104,3117 ----
    new->insertion = type;
    new->line_number = line_number;
    new->inhibited = inhibit_paragraph_indentation;
+ #ifdef RTF_OUTPUT
+   new->rtf_li = rtf_li;
+   new->rtf_fi = rtf_fi;
+   memcpy (new->rtf_tab_stops, rtf_tab_stops, sizeof (new->rtf_tab_stops));
+   new->rtf_n_tab_stops = rtf_n_tab_stops;
+   new->rtf_font = rtf_font;
+   new->rtf_fs = rtf_fs;
+ #endif
    new->next = insertion_stack;
    insertion_stack = new;
    insertion_level++;
***************
*** 2916,2921 ****
--- 3129,3142 ----
    inhibit_paragraph_indentation = temp->inhibited;
    filling_enabled = temp->filling_enabled;
    indented_fill = temp->indented_fill;
+ #ifdef RTF_OUTPUT
+   rtf_li = temp->rtf_li;
+   rtf_fi = temp->rtf_fi;
+   memcpy (rtf_tab_stops, temp->rtf_tab_stops, sizeof (rtf_tab_stops));
+   rtf_n_tab_stops = temp->rtf_n_tab_stops;
+   rtf_font = temp->rtf_font;
+   rtf_fs = temp->rtf_fs;
+ #endif
    free_and_clear (&(temp->item_function));
    insertion_stack = insertion_stack->next;
    free (temp);
***************
*** 3032,3044 ****
--- 3253,3276 ----
  	  current_enumval = ((current_enumval - 1) == 'z' ? 'a' : 'A');
  	  warning ("Lettering overflow, restarting at %c", current_enumval);
  	}
+ #ifdef RTF_OUTPUT
+       sprintf (temp, "%c.", current_enumval);
+ #else
        sprintf (temp, "%c. ", current_enumval);
+ #endif
      }
    else
+ #ifdef RTF_OUTPUT
+     sprintf (temp, "%d. ", current_enumval);
+ #else
      sprintf (temp, "%d. ", current_enumval);
+ #endif
  
    indent (output_column += (current_indent - strlen (temp)));
    add_word (temp);
+ #ifdef RTF_OUTPUT
+   inserts ("\\tab ");
+ #endif
    current_enumval++;
  }
  
***************
*** 3067,3074 ****
--- 3299,3314 ----
        filling_enabled = no_indent = 0;
        inhibit_paragraph_indentation = 1;
  
+ #ifdef RTF_OUTPUT
+       rtf_li = RTF_MENU_INDENTATION;
+       rtf_fi = -RTF_MENU_INDENTATION;
+       rtf_n_tab_stops = 1;
+       rtf_tab_stops[0] = rtf_li;
+       add_word ("* Menu:\n");
+ #else
        if (!no_headers)
  	add_word ("* Menu:\n");
+ #endif
  
        in_menu++;
        no_discard++;
***************
*** 3101,3106 ****
--- 3341,3350 ----
        if (type != format)
  	current_indent += default_indentation_increment;
  
+ #ifdef RTF_OUTPUT
+       rtf_font = RTF_FONT_FIXED_WIDTH;
+ #endif
+ 
        break;
  
      case table:
***************
*** 3125,3130 ****
--- 3369,3389 ----
  	      insertion_stack->item_function = savestring ("@bullet");
  	    }
  	}
+ 
+ #ifdef RTF_OUTPUT
+       if (type == itemize)
+ 	{
+ 	  rtf_li += RTF_ITEMIZE_INDENTATION_INCREMENT;
+ 	  rtf_fi = -RTF_ITEMIZE_INDENTATION_INCREMENT;
+ 	  rtf_n_tab_stops = 1;
+ 	  rtf_tab_stops[0] = rtf_li;
+ 	}
+       else
+ 	{
+ 	  rtf_li += RTF_TABLE_INDENTATION_INCREMENT;
+ 	  rtf_fi = 0;
+ 	}
+ #endif
        break;
  
      case enumerate:
***************
*** 3143,3148 ****
--- 3402,3414 ----
  	start_enumerating (atoi (enumeration_arg), ENUM_DIGITS);
        else
  	start_enumerating (*enumeration_arg, ENUM_ALPHA);
+ 
+ #ifdef RTF_OUTPUT
+       rtf_li += RTF_ENUMERATE_INDENTATION_INCREMENT;
+       rtf_fi = -RTF_ENUMERATE_INDENTATION_INCREMENT;
+       rtf_n_tab_stops = 1;
+       rtf_tab_stops[0] = rtf_li;
+ #endif
        break;
  
        /* Does nothing special in makeinfo. */
***************
*** 3356,3363 ****
--- 3622,3638 ----
  cm_bullet (arg)
       int arg;
  {
+ #ifdef RTF_OUTPUT
+   if (arg == START)
+     {
+     add_char ('*');
+       output_paragraph_offset--;
+       inserts ("{\\f2\\'B7}");
+     }
+ #else
    if (arg == START)
      add_char ('*');
+ #endif
  }
  
  cm_minus (arg)
***************
*** 3411,3421 ****
--- 3686,3704 ----
    if (arg == START)
      {
        in_fixed_width_font++;
+ #ifdef RTF_OUTPUT
+       vinserts ("{\\f%d ", RTF_FONT_FIXED_WIDTH);
+ #else
        add_char ('`');
+ #endif
      }
    else
      {
+ #ifdef RTF_OUTPUT
+       inserts ("}");
+ #else
        add_word ("'");
+ #endif
        in_fixed_width_font--;
      }
  }
***************
*** 3423,3435 ****
--- 3706,3740 ----
  cm_samp (arg)
       int arg;
  {
+ #ifdef RTF_OUTPUT
+   extern int printing_index;
+ 
+   if (printing_index)
+     return;
+ 
+   if (arg == START)
+     {
+       in_fixed_width_font++;
+       vinserts ("'{\\f%d ", RTF_FONT_FIXED_WIDTH);
+     }
+   else
+     {
+       inserts ("}'");
+       in_fixed_width_font--;
+     }
+ #else
    cm_code (arg);
+ #endif
  }
  
  cm_file (arg)
       int arg;
  {
+ #ifdef RTF_OUTPUT
+   cm_samp (arg);
+ #else
    cm_code (arg);
+ #endif
  }
  
  cm_kbd (arg)
***************
*** 3447,3460 ****
--- 3752,3783 ----
  cm_ctrl (arg, position)
       int arg, position;
  {
+ #ifdef RTF_OUTPUT
+   if (arg == START)
+     inserts ("{^");
+   else
+     inserts ("}");
+ #else
    if (arg == END)
      output_paragraph[position - 1] = CTL (output_paragraph[position]);
+ #endif
  }
  
  /* Small Caps in makeinfo just does all caps. */
  cm_sc (arg, start_pos, end_pos)
       int arg, start_pos, end_pos;
  {
+ #ifdef RTF_OUTPUT
+   extern int printing_index;
+ 
+   if (printing_index)
+     return;
+ 
+   if (arg == START)
+     vinserts ("{\\scaps\\fs16 "); /* FIXME */
+   else
+     inserts ("}");
+ #else
    if (arg == END)
      {
        while (start_pos < end_pos)
***************
*** 3464,3475 ****
--- 3787,3805 ----
  	  start_pos++;
  	}
      }
+ #endif
  }
  
  /* @var in makeinfo just uppercases the text. */
  cm_var (arg, start_pos, end_pos)
       int arg, start_pos, end_pos;
  {
+ #ifdef RTF_OUTPUT
+   if (arg == START)
+     inserts ("{\\i ");
+   else
+     inserts ("}");
+ #else
    if (arg == END)
      {
        while (start_pos < end_pos)
***************
*** 3479,3484 ****
--- 3809,3815 ----
  	  start_pos++;
  	}
      }
+ #endif
  }
  
  cm_dfn (arg, position)
***************
*** 3490,3496 ****
--- 3821,3831 ----
  cm_emph (arg)
       int arg;
  {
+ #ifdef RTF_OUTPUT
+   cm_var (arg);
+ #else
    add_char ('*');
+ #endif
  }
  
  cm_strong (arg, position)
***************
*** 3512,3517 ****
--- 3847,3855 ----
  cm_italic (arg, start, end)
       int arg, start, end;
  {
+ #ifdef RTF_OUTPUT
+   cm_emph (arg, start, end);
+ #endif
  }
  
  /* Current text is highlighted. */
***************
*** 3575,3595 ****
--- 3913,3967 ----
    int starting_pos, ending_pos;
    char *temp;
  
+ #ifdef RTF_OUTPUT
+   int big_heading = in_non_scrolling_region;
+ 
+   get_rest_of_line (&temp);
+   if (in_non_scrolling_region)
+     {
+       inserts ("${\\footnote ");
+       execute_string ("%s", temp);
+       inserts ("}\n");
+       in_non_scrolling_region--;
+       close_paragraph_with_lines (0);
+     }
+   else
+     close_paragraph ();
+ #else
    close_paragraph ();
+ #endif
    filling_enabled =  indented_fill = 0;
    old_no_indent = no_indent;
    no_indent = 1;
+ #ifdef RTF_OUTPUT
+ #else
    get_rest_of_line (&temp);
+ #endif
+ 
+ #ifdef RTF_OUTPUT
+   if (big_heading)
+     inserts ("\\pard{\\fs24\\b ");
+   else
+     inserts ("\\pard{\\b ");
+ #endif
  
    starting_pos = output_position + output_paragraph_offset;
+ #ifdef RTF_OUTPUT
+   execute_string ("%s", temp);
+ #else
    execute_string ("%s\n", temp);
+ #endif
    ending_pos = output_position + output_paragraph_offset;
    free (temp);
  
    len = (ending_pos - starting_pos) - 1;
+ #ifdef RTF_OUTPUT
+   inserts ("}");
+ #else
    for (i = 0; i < len; i++)
      add_char (with_char);
    insert ('\n');
+ #endif
    close_paragraph ();
    filling_enabled = 1;
    no_indent = old_no_indent;
***************
*** 4082,4089 ****
--- 4454,4464 ----
    up = get_node_token ();
  
    no_indent = 1;
+ #ifdef RTF_OUTPUT
+ #else
    if (!no_headers)
      add_word_args ("\037\nFile: %s,  Node: %s", pretty_output_filename, node);
+ #endif
  
    /* Check for defaulting of this node's next, prev, and up fields. */
    defaulting = ((strlen (next) == 0) &&
***************
*** 4203,4208 ****
--- 4578,4586 ----
  	}
      }
  
+ #ifdef RTF_OUTPUT
+   start_topic (node, next, prev, up);
+ #else
    if (!no_headers)
      {
        if (*next)
***************
*** 4214,4219 ****
--- 4592,4598 ----
        if (*up)
  	add_word_args (",  Up: %s", up);
      }
+ #endif
  
    close_paragraph ();
    no_indent = 0;
***************
*** 4610,4621 ****
--- 4989,5009 ----
  
  		  split_filename = (char *) xmalloc
  		    (10 + strlen (root_pathname) + strlen (root_filename));
+ #if defined (__MSDOS__)
+ 		  sprintf
+ 		    (split_filename,
+ 		      "%s%s.%d", root_pathname, root_filename, which_file);
+ 
+ 		  fd = open
+ 		    (split_filename, O_WRONLY | O_TRUNC | O_CREAT | O_BINARY, 0666);
+ #else /* __MSDOS__ */
  		  sprintf
  		    (split_filename,
  		     "%s%s-%d", root_pathname, root_filename, which_file);
  
  		  fd = open
  		    (split_filename, O_WRONLY | O_TRUNC | O_CREAT, 0666);
+ #endif /* __MSDOS__ */
  
  		  if ((fd < 0) ||
  		      (write (fd, the_header, header_size) != header_size) ||
***************
*** 4636,4643 ****
--- 5024,5036 ----
  		      indirect_info += strlen (indirect_info);
  		    }
  
+ #if defined (__MSDOS__)
+ 		  sprintf (indirect_info, "%s.%d: %d\n",
+ 			   root_filename, which_file, file_top);
+ #else /* __MSDOS__ */
  		  sprintf (indirect_info, "%s-%d: %d\n",
  			   root_filename, which_file, file_top);
+ #endif /* __MSDOS__ */
  
  		  free (split_filename);
  		  indirect_info += strlen (indirect_info);
***************
*** 4651,4657 ****
--- 5044,5054 ----
      /* We have sucessfully created the subfiles.  Now write out the
         original again.  We must use `output_stream', or
         write_tag_table_indirect () won't know where to place the output. */
+ #ifdef __MSDOS__
+     output_stream = fopen (filename, "wb");
+ #else
      output_stream = fopen (filename, "w");
+ #endif
      if (!output_stream)
        {
  	perror (filename);
***************
*** 4816,4821 ****
--- 5213,5275 ----
      return (nodename);
  }
  
+ #ifdef RTF_OUTPUT
+ process_menu_line ()
+ {
+   int i, j;
+   char *nodename;
+ 
+   if (strncmp (&input_text[input_text_offset],
+ 	       menu_starter,
+ 	       strlen (menu_starter)) != 0)
+     return (0);
+   else
+     input_text_offset += strlen (menu_starter);
+ 
+   get_until_in_line (":", &nodename);
+   if (curchar () == ':')
+     input_text_offset++;
+   canon_white (nodename);
+ 
+   add_char ('i');
+   output_paragraph_offset--;
+ 
+   inserts ("{\\uldb ");
+   execute_string ("%s", nodename);
+   inserts ("}{\\v ");
+ 
+   if (curchar () == ':')
+     {
+       input_text_offset++;
+       i = strlen (nodename);
+       if (i && nodename[i - 1] == ':')
+ 	nodename[i - 1] = '\0';
+       insert_context_string (nodename);
+       free (nodename);
+       inserts ("}\\tab ");
+       skip_whitespace ();
+     }
+   else
+     {
+       free (nodename);
+       get_rest_of_line (&nodename);
+       i = strlen (nodename);
+       backup_input_pointer ();
+       isolate_nodename (nodename);
+       insert_context_string (nodename);
+       inserts ("}\\tab ");
+       j = strlen (nodename);      
+       if (i > j)
+ 	{
+ 	  char *description = nodename + j + 1;
+ 	  canon_white (description);
+ 	  execute_string ("%s", description);
+ 	}
+       free (nodename);
+     }
+ }
+ #endif
+ 
  static void
  isolate_nodename (nodename)
       char *nodename;
***************
*** 4909,4915 ****
--- 5363,5372 ----
        arg4 = get_xref_token ();
        arg5 = get_xref_token ();
  
+ #ifdef RTF_OUTPUT
+ #else
        add_word_args ("%s", px_ref_flag ? "*note " : "*Note ");
+ #endif
  
        if (*arg5 || *arg4)
  	{
***************
*** 4925,4931 ****
--- 5382,5396 ----
  	  else
  	    node_name = arg2;
  
+ #ifdef RTF_OUTPUT
+ 	  inserts ("{\\uldb ");
+ 	  execute_string ("%s", node_name);
+ 	  inserts ("}{\\v ");
+ 	  insert_context_string (arg1);
+ 	  vinserts (" @%s.hlp}", arg4);
+ #else
  	  execute_string ("%s: (%s)%s", node_name, arg4, arg1);
+ #endif
  	  return;
  	}
        else
***************
*** 4933,4949 ****
--- 5398,5452 ----
  
        if (*arg3)
  	{
+ #ifdef RTF_OUTPUT
+ 	  if (!*arg2)
+ 	    {
+ 	      inserts ("{\\uldb ");
+ 	      execute_string ("%s", arg3);
+ 	      inserts ("}{\\v ");
+ 	      insert_context_string (arg1);
+ 	      inserts ("}");
+ 	    }
+ 	  else
+ 	    {
+ 	      inserts ("{\\uldb ");
+ 	      execute_string ("%s", arg2);
+ 	      inserts ("}{\\v ");
+ 	      insert_context_string (arg1);
+ 	      inserts ("}");
+ 	    }
+ #else
  	  if (!*arg2)
  	    execute_string ("%s: %s", arg3, arg1);
  	  else
  	    execute_string ("%s: %s", arg2, arg1);
+ #endif
  	  return;
  	}
  
+ #ifdef RTF_OUTPUT
+       if (*arg2)
+ 	{
+ 	  inserts ("{\\uldb ");
+ 	  execute_string ("%s", arg2);
+ 	  inserts ("}{\\v ");
+ 	  insert_context_string (arg1);
+ 	  inserts ("}");
+ 	}
+       else
+ 	{
+ 	  inserts ("{\\uldb ");
+ 	  execute_string ("%s", arg1);
+ 	  inserts ("}{\\v ");
+ 	  insert_context_string (arg1);
+ 	  inserts ("}");
+ 	}
+ #else
        if (*arg2)
  	execute_string ("%s: %s", arg2, arg1);
        else
  	execute_string ("%s::", arg1);
+ #endif
      }
    else
      {
***************
*** 5572,5578 ****
--- 6075,6085 ----
  		      else
  			execute_string ("%s", item_func);
  		    }
+ #ifdef RTF_OUTPUT
+ 		  inserts ("\\tab ");
+ #else
  		  insert (' ');
+ #endif
  		  output_column++;
  		}
  	      else
***************
*** 5598,5603 ****
--- 6105,6114 ----
  	case ftable:
  	case vtable:
  	  {
+ #ifdef RTF_OUTPUT
+ 	    int old_rtf_li = rtf_li;
+ 	    rtf_li -= RTF_TABLE_INDENTATION_INCREMENT;
+ #endif
  	    /* Get rid of extra characters. */
  	    kill_self_indent (-1);
  
***************
*** 5645,5650 ****
--- 6156,6164 ----
  	       do the indenting of it for you. */
  	    close_single_paragraph ();
  	    indented_fill = filling_enabled = 1;
+ #ifdef RTF_OUTPUT
+ 	    rtf_li = old_rtf_li;
+ #endif
  	  }
  	}
        free (rest_of_line);
***************
*** 6673,6678 ****
--- 7187,7197 ----
        new->code = tem->code;
        new->defining_line = line_number - 1;
        the_indices[which] = new;
+ #ifdef RTF_OUTPUT
+       inserts ("K{\\footnote K ");
+       save_insert (index_entry);
+       inserts ("}\n");
+ #endif
      }
  }
  
***************
*** 6908,6914 ****
--- 7427,7438 ----
        if (fill_column > 40 && output_column < 40)
  	indent (40 - output_column);
  
+ #ifdef RTF_OUTPUT
+       inserts ("\\tab ");
+       execute_string ("@xref{%s}.\n", index->node);
+ #else
        execute_string ("%s.\n", index->node);
+ #endif
  
        line_number = real_line_number;
        flush_output ();
***************
*** 7547,7549 ****
--- 8071,8229 ----
      }
    return ((char *)NULL);
  }
+ 
+ #ifdef RTF_OUTPUT
+ /* **************************************************************** */
+ /*                                                                  */
+ /*                           RTF-Stuff                              */
+ /*                                                                  */
+ /* **************************************************************** */
+ 
+ inserts (s)
+      char *s;
+ {
+   while (*s)
+     insert (*s++);
+ }
+ 
+ #if defined (HAVE_VARARGS_H) && defined (HAVE_VSPRINTF)
+ 
+ vinserts (va_alist)
+      va_dcl
+ {
+   char buffer[1000];
+   char *format;
+   va_list args;
+ 
+   va_start (args);
+   format = va_arg (args, char *);
+   vsprintf (buffer, format, args);
+   va_end (args);
+   inserts (buffer);
+ }
+ 
+ #else /* !(HAVE_VARARGS_H && HAVE_VSPRINTF) */
+ 
+ vinserts (format, arg1, arg2, arg3, arg4, arg5)
+      char *format;
+ {
+   char buffer[1000];
+   sprintf (buffer, format, arg1, arg2, arg3, arg4, arg5);
+   inserts (buffer);
+ }
+ 
+ #endif /* !(HAVE_VARARGS_H && HAVE_VSPRINTF) */
+ 
+ save_insert (s)
+      char *s;
+ {
+   while (*s)
+     {
+       if (*s == '{' || *s == '}' || *s == '\\' || ! isascii (*s))
+ 	vinserts ("\\'%02X", (int) (*s & 0xff));
+       else
+ 	insert (*s);
+       s++;
+     }
+ }
+ 
+ make_context_string (context_string, text)
+      char *context_string;
+      char *text;
+ {
+   char *cp = context_string;
+   if (text)
+     {
+       while (*text)
+ 	{
+ 	  if (*text == '\n')
+ 	    {
+ 	      *cp++ = '_';
+ 	      text++;
+ 	      while (*text && isspace(*text))
+ 		text++;
+ 	      if (*text == 0)
+ 		break;
+ 	    }
+ 	  if (! isalnum (*text))
+ 	    *cp = '_';
+ 	  else
+ 	    *cp = *text;
+ 	  cp++;
+ 	  text++;
+ 	}
+     }
+   *cp = 0;
+ }
+ 
+ insert_context_string (text)
+      char *text;
+ {
+   char context_string[ 8192 ];
+   make_context_string (context_string, text);
+   inserts (context_string);
+ }
+ 
+ insert_hot_spot (text, node)
+      char *text;
+      char *node;
+ {
+   char context_string[ 8192 ];
+   if (node == 0)
+     make_context_string (context_string, text);
+   else
+     make_context_string (context_string, node);
+   inserts ("{\\uldb ");
+   if (text == 0 || *text == 0)
+     {
+       if (node)
+ 	save_insert (node);
+     }
+   else
+     save_insert (text);
+   inserts ("}{\\v ");
+   save_insert (context_string);
+   inserts ("}");
+ }
+ 
+ start_rtf_file ()
+ {
+   fprintf (output_stream, "\
+ {\\rtf1\\windows \\deff0{\\fonttbl\n\
+ {\\f0\\fswiss MS Sans Serif;}{\\f1\\fmodern Courier New;}{\\f2\\ftech Symbol;}}\\fs20\n");
+ }
+ 
+ end_rtf_file ()
+ {
+   fprintf (output_stream, "}\n");
+ }
+ 
+ start_topic (node, next, prev, up)
+      char *node;
+      char *next;
+      char *prev;
+      char *up;
+ {
+   char context_string[ 8192 ];
+ 
+   in_non_scrolling_region++;
+ 
+   inserts ("\\page\\pard Node: {\\b ");
+   save_insert (node);
+   inserts ("}, \\keepn Next: ");
+   insert_hot_spot (0, next);
+   inserts (", Prev: ");
+   insert_hot_spot (0, prev);
+   inserts (", Up: ");
+   insert_hot_spot (0, up);
+   inserts ("\\line\n");
+ 
+   make_context_string (context_string, node);
+   inserts ("K{\\footnote K ");
+   save_insert (node);
+   inserts ("}\n");
+   inserts ("#{\\footnote ");
+   save_insert (context_string);
+   inserts ("}\n");
+ }
+ #endif
